# -*- cperl -*- #

=head1 NAME

Xen::Host - A module for working with a Xen instance

=head1 SYNOPSIS

=for example begin

    #!/usr/bin/perl -w

    use Xen::Host;
    use strict;

    my $host = Xen::Host->new( name => 'test' );
    $host->shutdown();

=for example end


=head1 DESCRIPTION

  TODO

=cut


package Xen::Host;

use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);


require Exporter;
require AutoLoader;

@ISA	= qw(Exporter AutoLoader);
@EXPORT = ();

($VERSION) = '$Revision: 1.10 $' =~ m/Revision:\s*(\S+)/;



#
#  Standard modules which we require.
#
use strict;
use warnings;

#
#  Allow this object to return a string representation of itself.
# 
use overload '""'  => \&stringify ;

#
#  Standard modules we use.
#
use Socket;


#
#  Xen-perl modules we use.
#
use Xen::Monitor;





=head2 new

  Create a new instance of this object.

=for example begin

    #!/usr/bin/perl -w

    use Xen::Host;

    my $host = Xen::Host->new( name => 'test' );

=for example end

  The name given will be used to determine which configuration file
 is used in /etc/xen/

=cut

sub new
{
    my ( $proto, %supplied ) = (@_);
    my $class = ref($proto) || $proto;

    my $self  = {};

    #
    #  Allow user supplied values to override our defaults
    #
    foreach my $key ( keys %supplied )
    {
	$self->{ lc $key } = $supplied{ $key };
    }

    bless ($self, $class);
    return $self;

}


=head2 getName

  Return the name of this object, this is the name the object was
 given in the <new> function.

=cut

sub getName
{
    my ($class) = ( @_ );

    my $name = $class->{'name'};
    
    # Restrict names to valid characters.
    $name =~ tr[A-Za-z0-9_.]||cd if defined( $name );

    return( $name );
}



=head2 getMemory

  Return the amount of memory the host is using.

=cut

sub getMemory
{
    my ($class)	= ( @_ );

    my $memory	= 0;
    my $name	= $class->getName();

    die "No name" unless defined( $name );

    #
    #  If the server is running we can get the memory via "xm list".
    #
    if ( $class->isRunning() )
    {
	my $cmd    = "xm list  --long $name 2>/dev/null";

	open( RUNNING, $cmd . "|" );
	
	foreach my $line ( <RUNNING> )
	{
	    if ( $line =~ /\(memory[ \t]([0-9]+)\)/ )
	    {
		$memory= $1;
	    }
	}
	close( RUNNING );
    }
    else
    {
	#
	# Not running, read the memory from the cfg file.
	#
	if ( -e "/etc/xen/$name.cfg" )
	{
	    open( CFG, "<", "/etc/xen/$name.cfg" );
	    while( <CFG> )
	    {
		my $line = $_;

		if ( $line =~ /memory[ \t]*=[ \t]*([0-9]+)/ )
		{
		    $memory = $1;
		}
	    }
	    close( CFG );
	      
	}
    }
    return( $memory );
}



=head2 isPaused

  Return 1 if the given machine is paused.

=cut

sub isPaused
{
    my ($class) = ( @_ );

    my $name   = $class->getName();
    die "No name" unless defined( $name );


    my $cmd    = "xm list  --long $name 2>/dev/null";
    my $paused = 0;

    open( RUNNING, $cmd . "|" );
    foreach my $line ( <RUNNING> )
    {
	if ( $line =~ /\(state (.*)\)/ )
	{
	    my $state = $1;
	    if ( $state =~ /p/ )
	    {
		$paused = 1;
	    }
	}
    }
    close( RUNNING );

    return( $paused );
}





=head2 isRunning

  Return 1 if the given machine is running.

=cut

sub isRunning
{
    my ($class) = ( @_ );

    my $name	= $class->getName();
    die "No name" unless defined( $name );


    my $cmd	= "xm list  --long $name 2>/dev/null";
    my $running	= 0;

    open( RUNNING, $cmd . "|" );
    foreach my $line ( <RUNNING> )
    {
	if ( $line =~ /\(state (.*)\)/ )
	{
	    my $state = $1;

	    #
	    #  If we have a state then we can assume we're running
	    # unless:
	    #
	    #  1. it is pasued.
	    #  2. it is crashed.
	    #  3. it is dying.
	    #
	    $running = 1;

	    if ( $state =~ /[pcd]/ )
	    {
		# paused / crashed / dying
		$running = 0;
	    }
	}
    }
    close( RUNNING );

    return( $running );
}


=head2 pause

  Pause the instance.

=cut

sub pause
{
    my ($class) = ( @_ );

    my $name = $class->getName();
    die "No name" unless defined( $name );

    my $cmd  = "xm pause $name 2>/dev/null";
    `$cmd`;
}



=head2 unpause

  Unpause the instance.

=cut

sub unpause
{
    my ($class) = ( @_ );

    my $name = $class->getName();
    die "No name" unless defined( $name );

    my $cmd  = "xm unpause $name 2>/dev/null";
    print `$cmd`;
}


=head2 start

 Start the instance.

=cut

sub start
{
    my ($class) = ( @_ );

    my $name = $class->getName();
    die "No name" unless defined( $name );

    my $cmd  = "xm create $name.cfg 2>/dev/null";
    `$cmd`;
}


=head2 reboot

  Reboot the given instance 

=cut 

sub reboot
{
    my ($class) = ( @_ );

    my $name = $class->getName();
    die "No name" unless defined( $name );

    my $cmd  = "xm reboot $name 2>/dev/null";
    `$cmd`;
}


=head2 stop

 Stop the instance via a shutdown.

=cut 

sub stop
{
    my ($class) = ( @_ );

    my $name = $class->getName();
    die "No name" unless defined( $name );

    my $cmd  = "xm shutdown $name 2>/dev/null";
    `$cmd`;
}


=head2 printInfo

  This function is useful for debugging and just prints out
 some information about the current host.

=cut

sub printInfo
{
    my ($class) = (@_);
    print $class;
}



=head2 stringify

  This function returns a string representation of this
 object.

  This is useful for debug purposes:

=for example start

    my $host = Xen::Host->new( name => 'one.my.flat' );
    print $host;

=for example end

=cut

sub stringify
{
    my ($class) = (@_);

    my $name = $class->getName();
    die "No name" unless defined( $name );

    my $output = "";

    $output .= "Name: $name\n";
    $output .= "Memory: " . $class->getMemory()    . "\n";
    $output .= "Uptime: " . $class->getUptime()    . "\n";
    $output .= "IP: "     . $class->getIPAddress() . "\n";

    if ( $class->isRunning() )
    {
        $output .= "State: Running\n";
        $output .= "MAC: " . $class->getMAC() . "\n";
    }
    elsif ( $class->isPaused() )
    {
        $output .= "State: Paused\n";
        $output .= "MAC: " . $class->getMAC() . "\n";
    }
    else
    {
        $output .= "State: Off\n";
    }

    return( $output );
}



=head2 getUptime

  Get the uptime of this instance.

=cut 

sub getUptime
{
    my ($class) = (@_);

    my $name = $class->getName();
    die "No name" unless defined( $name );

    if ( $class->isRunning() )
    {
	my $seconds = "";
	my $cmd	    = "xm list  --long $name 2>/dev/null";

	open( RUNNING, $cmd . "|" );
	
	foreach my $line ( <RUNNING> )
	{
	    if ( $line =~ /\(up_time[ \t]*([0-9]+)/ )
	    {
		$seconds= $1;
	    }
	}
	close( RUNNING );

	my $days  = int($seconds/(24*60*60));
	my $hours = ($seconds/(60*60))%24;
	my $mins  = ($seconds/60)%60;
	my $secs  = $seconds%60; 

	if ( length( $hours ) < 2 ) { $hours = "0" . $hours ; }
	if ( length( $mins ) < 2 )  { $mins  = "0" . $mins ; }
	if ( length( $secs ) < 2 )  { $secs  = "0" . $secs ; }

	return( "$days days $hours:$mins:$secs" );
    }
    else
    {
	return( "" );
    }
}


=head2 getIPAddress

  Find and return the IP address of the host.
 
  This is a lotech solution which attempts to find it via DNS..

=cut

sub getIPAddress
{
    my ($class) = (@_);

    my $name = $class->getName();
    die "No name" unless defined( $name );

    my $aton = inet_aton($name);
    if ( !defined( $aton ) ) { return( "" ); }
    my $ip   = inet_ntoa($aton);
    if ( !defined( $ip ) ) { return( "" ); }

    return( $ip );
}




=head2 getMAC

  Return the MAC address of a running instance.

  This function makes no sense when the instance is not running.

=cut

sub getMAC 
{
    my ($class) = (@_);

    my $name = $class->getName();
    die "No name" unless defined( $name );

    my @data = "";

    open( RUNNING, "xm list --long $name|" );
    foreach my $line ( <RUNNING> )
    {
	next if ( !defined( $line ) ) ;

	$line =~ s/\r\n//g;
	push @data, $line ;
    }
    close( RUNNING );

    foreach my $line ( @data )
    {
	if ( $line =~ /\(mac ([^\)]+)/ )
	{
	    return $1;
	}
    }

    return( "00:00:00:00:00:00" );
     
}




=head2 getRX

  Return the amount of data recieved by this host.

  This function makes no sense when the instance is not running.

=cut

sub getRX
{
    my ($class) = (@_);

    my $name = $class->getName();
    die "No name" unless defined( $name );

    my @data = "";

    open( RUNNING, "xm list --long $name|" );
    foreach my $line ( <RUNNING> )
    {
	next if ( !defined( $line ) ) ;

	$line =~ s/\r\n//g;
	push @data, $line ;
    }
    close( RUNNING );

    #
    #  Find the domain ID.
    #
    my $id = undef;
    foreach my $line ( @data )
    {
	if ( $line =~ /\(domid ([^\)]+)/ )
	{
	    $id = $1;
	}
    }

    if (!defined ($id ) )
    {
	return 0;
    }

    #
    #  Now read the networking details.
    #
    my $count = 0;
    open( NET, "<", "/proc/net/dev" );
    foreach my $line ( <NET> )
    {
	if ( $line =~ /^vif$id/ )
	{
	    if ( $line =~ /(.*):([ \t]+)([0-9]+)/ )
	    {
		$count = $3;
	    }
	}
    }
    close( NET );
    return( $count );
    
}




=head2 getTX

  Return the amount of data transmitted by this host.

  This function makes no sense when the instance is not running.

=cut

sub getTX
{
    my ($class) = (@_);

    my $name = $class->getName();
    die "No name" unless defined( $name );

    my @data = "";

    open( RUNNING, "xm list --long $name|" );
    foreach my $line ( <RUNNING> )
    {
	next if ( !defined( $line ) ) ;

	$line =~ s/\r\n//g;
	push @data, $line ;
    }
    close( RUNNING );

    #
    #  Find the domain ID.
    #
    my $id = undef;
    foreach my $line ( @data )
    {
	if ( $line =~ /\(domid ([^\)]+)/ )
	{
	    $id = $1;
	}
    }

    if (!defined ($id ) )
    {
	return 0;
    }

    #
    #  Now read the networking details.
    #
    my $count = 0;
    open( NET, "<", "/proc/net/dev" );
    foreach my $line ( <NET> )
    {
	if ( $line =~ /^vif$id/ )
	{
	    if ( $line =~ /([0-9]+)[ \t]+([0-9]+)[ \t]+([0-9]+)[ \t]+([0-9]+)[ \t]+([0-9]+)[ \t]+([0-9]+)[ \t]+([0-9]+)[ \t]+([0-9]+)$/ )
	    {
		$count = $1;
	    }
	}
    }
    close( NET );
    return( $count );
    
}




=head1 LICENSE

Copyright (c) 2005 by Steve Kemp.  All rights reserved.

This module is free software;
you can redistribute it and/or modify it under
the terms of the GNU General Public License.

=cut


1;
